@@ -3,15 +3,20 @@ Base script class for spiders crawl managers.
 """
 import abc
 import json
+import asyncio
 import logging
 from copy import deepcopy
 from argparse import Namespace
-from typing import Optional, List, Tuple, Dict, NewType, cast, Generator, Any
+from typing import Optional, List, Tuple, Dict, NewType, cast, Generator, Any, AsyncGenerator
 
 from bloom_filter import BloomFilter
-from typing_extensions import TypedDict, NotRequired
+from typing_extensions import TypedDict, NotRequired, Protocol
 
-from shub_workflow.script import JobKey, JobDict
+from shub_workflow.script import (
+    JobKey,
+    JobDict,
+    BaseLoopScriptAsyncMixin,
+)
 from shub_workflow.base import WorkFlowManager
 from shub_workflow.utils import hashstr
 
@@ -96,9 +101,10 @@ class CrawlManager(WorkFlowManager):
 
     def schedule_spider_with_jobargs(
         self,
-        job_args_override: JobParams,
+        job_args_override: Optional[JobParams] = None,
         spider: Optional[str] = None,
     ) -> Optional[JobKey]:
+        job_args_override = job_args_override or {}
         spider = spider or self.spider
         if spider is not None:
             schedule_args: Dict[str, Any] = json.loads(self.args.spider_args)
@@ -110,27 +116,12 @@ class CrawlManager(WorkFlowManager):
             job_settings_override = schedule_args.get("job_settings", None)
             schedule_args["job_settings"] = self.get_job_settings(job_settings_override)
             schedule_args["units"] = schedule_args.get("units", self.args.units)
-            jobkey = super().schedule_spider(spider, **schedule_args)
+            jobkey = self.schedule_spider(spider, **schedule_args)
             if jobkey is not None:
                 self._running_job_keys[jobkey] = spider, job_args_override
             return jobkey
         return None
 
-    def schedule_spider(
-        self,
-        spider: Optional[str] = None,
-        tags: Optional[List[str]] = None,
-        units: Optional[int] = None,
-        project_id: Optional[int] = None,
-        **kwargs,
-    ) -> Optional[JobKey]:
-        job_args_override = kwargs.pop("job_args_override", None)
-        if spider is None or job_args_override is not None:
-            job_args_override = job_args_override or {}
-            job_args_override.update(kwargs)
-            return self.schedule_spider_with_jobargs(job_args_override, spider)
-        return super().schedule_spider(spider, **kwargs)
-
     def check_running_jobs(self) -> Dict[JobKey, str]:
         outcomes = {}
         running_job_keys = list(self._running_job_keys)
@@ -157,7 +148,7 @@ class CrawlManager(WorkFlowManager):
         if outcomes:
             return False
         if not self._running_job_keys:
-            self.schedule_spider()
+            self.schedule_spider_with_jobargs()
         return True
 
     def resume_running_job_hook(self, job: JobDict):
@@ -186,14 +177,46 @@ class PeriodicCrawlManager(CrawlManager):
     def workflow_loop(self) -> bool:
         self.check_running_jobs()
         if not self._running_job_keys:
-            self.schedule_spider()
+            self.schedule_spider_with_jobargs()
         return True
 
     def on_close(self):
         pass
 
 
-class GeneratorCrawlManager(CrawlManager):
+class GeneratorCrawlManagerProtocol(Protocol):
+
+    _running_job_keys: Dict[JobKey, Tuple[str, JobParams]]
+    _jobuids: BloomFilter
+    spider: Optional[str]
+
+    @property
+    @abc.abstractmethod
+    def max_running_jobs(self) -> int:
+        ...
+
+    @abc.abstractmethod
+    def _workflow_step_gen(self, max_next_params: int) -> Generator[Tuple[str, Optional[JobKey]], None, None]:
+        ...
+
+    @abc.abstractmethod
+    def check_running_jobs(self) -> Dict[JobKey, str]:
+        ...
+
+    @abc.abstractmethod
+    def schedule_spider_with_jobargs(
+        self,
+        job_args_override: Optional[JobParams] = None,
+        spider: Optional[str] = None,
+    ) -> Optional[JobKey]:
+        ...
+
+    @abc.abstractmethod
+    def get_job_settings(self, override: Optional[Dict[str, str]] = None) -> Dict[str, str]:
+        ...
+
+
+class GeneratorCrawlManager(CrawlManager, GeneratorCrawlManagerProtocol):
     """
     Schedule a spider periodically, each time with different parameters yielded by a generator, until stop.
     Number of simultaneos spider jobs will be limited by max running jobs (see WorkFlowManager).
@@ -210,7 +233,7 @@ class GeneratorCrawlManager(CrawlManager):
         self.__parameters_gen = self.set_parameters_gen()
         self.__additional_jobs: List[FullJobParams] = []
         self.__next_job_seq = 1
-        self.__jobuids = BloomFilter(max_elements=self.MAX_TOTAL_JOBS, error_rate=0.001)
+        self._jobuids = BloomFilter(max_elements=self.MAX_TOTAL_JOBS, error_rate=0.001)
 
     def bad_outcome_hook(self, spider: str, outcome: str, job_args_override: JobParams, jobkey: JobKey):
         pass
@@ -240,7 +263,7 @@ class GeneratorCrawlManager(CrawlManager):
             jobseq_tag = f"JOBSEQ={jobseq:010d}.r{repn + 1}"
         tags.append(jobseq_tag)
 
-    def _workflow_step_gen(self, max_next_params: int) -> Generator[JobKey, None, None]:
+    def _workflow_step_gen(self, max_next_params: int) -> Generator[Tuple[str, Optional[JobKey]], None, None]:
         new_params = []
         for i in range(max_next_params):
             try:
@@ -256,19 +279,18 @@ class GeneratorCrawlManager(CrawlManager):
             assert spider is not None, "No spider set."
             spider_args = get_spider_args_from_params(next_params)
             jobuid = self.get_job_unique_id({"spider": spider, "spider_args": spider_args})
-            if jobuid not in self.__jobuids:
+            if jobuid not in self._jobuids:
                 self.__add_jobseq_tag(next_params)
-                jobid = self.schedule_spider(spider=spider, job_args_override=next_params)
-                if jobid is not None:
-                    self.__jobuids.add(jobuid)
-                    yield jobid
+                yield jobuid, self.schedule_spider_with_jobargs(next_params, spider)
 
     def workflow_loop(self) -> bool:
         self.check_running_jobs()
         max_next_params = self.max_running_jobs - len(self._running_job_keys)
         retval = False
-        for _ in self._workflow_step_gen(max_next_params):
+        for jobuid, jobid in self._workflow_step_gen(max_next_params):
             retval = True
+            if jobid is not None:
+                self._jobuids.add(jobuid)
         return retval or bool(self._running_job_keys)
 
     @staticmethod
@@ -283,13 +305,13 @@ class GeneratorCrawlManager(CrawlManager):
     def resume_running_job_hook(self, job: JobDict):
         super().resume_running_job_hook(job)
         jobuid = self.get_job_unique_id(job)
-        self.__jobuids.add(jobuid)
+        self._jobuids.add(jobuid)
         self.__next_job_seq = max(self.__next_job_seq, get_jobseq(job["tags"])[0] + 1)
 
     def resume_finished_job_hook(self, job: JobDict):
         jobuid = self.get_job_unique_id(job)
-        if jobuid not in self.__jobuids:
-            self.__jobuids.add(jobuid)
+        if jobuid not in self._jobuids:
+            self._jobuids.add(jobuid)
         self.__next_job_seq = max(self.__next_job_seq, get_jobseq(job["tags"])[0] + 1)
 
     def resume_workflow(self):
@@ -298,3 +320,47 @@ class GeneratorCrawlManager(CrawlManager):
 
     def on_close(self):
         pass
+
+
+class AsyncSchedulerCrawlManagerMixin(
+    BaseLoopScriptAsyncMixin, GeneratorCrawlManagerProtocol
+):
+    async def _async_workflow_step_gen(self, max_next_params: int) -> AsyncGenerator[Tuple[str, JobKey], None]:
+        jobuids_cors = list(self._workflow_step_gen(max_next_params))
+        if jobuids_cors:
+            jobuids, cors = zip(*jobuids_cors)
+            results: List[JobKey] = await asyncio.gather(*cors)
+            for jobuid, jobid in zip(jobuids, results):
+                yield jobuid, jobid
+
+    async def workflow_loop(self) -> bool:  # type: ignore
+        self.check_running_jobs()
+        max_next_params = self.max_running_jobs - len(self._running_job_keys)
+        retval = False
+        async for jobuid, jobid in self._async_workflow_step_gen(max_next_params):
+            retval = True
+            if jobid is not None:
+                self._jobuids.add(jobuid)
+        return retval or bool(self._running_job_keys)
+
+    async def schedule_spider_with_jobargs(  # type: ignore
+        self,
+        job_args_override: Optional[JobParams] = None,
+        spider: Optional[str] = None,
+    ) -> Optional[JobKey]:
+        job_args_override = job_args_override or {}
+        spider = spider or self.spider
+        assert spider is not None
+        schedule_args: Dict[str, Any] = json.loads(self.args.spider_args)
+        spider_args = get_spider_args_from_params(job_args_override)
+        schedule_args.update(job_args_override)
+        schedule_args.pop("spider_args", None)
+        schedule_args.update(spider_args)
+
+        job_settings_override = schedule_args.get("job_settings", None)
+        schedule_args["job_settings"] = self.get_job_settings(job_settings_override)
+        schedule_args["units"] = schedule_args.get("units", self.args.units)
+        jobkey = await self.async_schedule_spider(spider, **schedule_args)
+        if jobkey is not None:
+            self._running_job_keys[jobkey] = spider, job_args_override
+        return jobkey
