@@ -4,11 +4,13 @@ Implements common methods for ScrapyCloud scripts.
 import os
 import abc
 import json
+import asyncio
 import logging
 import time
+from functools import partial
 import subprocess
 from argparse import ArgumentParser, Namespace
-from typing import List, NewType, Optional, Tuple, Generator, Dict, Union, Any
+from typing import List, NewType, Optional, Tuple, Generator, Dict, Union, Any, AsyncGenerator, Awaitable, cast
 from typing_extensions import TypedDict, NotRequired, Protocol
 
 from scrapinghub import ScrapinghubClient, DuplicateJobError
@@ -42,6 +44,9 @@ class JobDict(TypedDict):
 
 
 class ArgumentParserScriptProtocol(Protocol):
+
+    args: Namespace
+
     @abc.abstractmethod
     def add_argparser_options(self):
         ...
@@ -137,6 +142,10 @@ class BaseScriptProtocol(ArgumentParserScriptProtocol, Protocol):
     def finish(self, jobkey: Optional[JobKey] = None, close_reason: Optional[str] = None):
         ...
 
+    @abc.abstractmethod
+    def _make_children_tags(self, tags: Optional[List[str]]) -> Optional[List[str]]:
+        ...
+
 
 class BaseScript(ArgumentParserScript, BaseScriptProtocol):
 
@@ -160,8 +169,7 @@ class BaseScript(ArgumentParserScript, BaseScriptProtocol):
         self.__flow_tags.append(tag)
         self.add_job_tags(tags=[tag])
 
-    @staticmethod
-    def generate_flow_id() -> str:
+    def generate_flow_id(self) -> str:
         raise NotImplementedError("generate_flow_id() must be implemented if flow_id_required is True.")
 
     def set_flow_id_name(self, args: Namespace):
@@ -305,7 +313,7 @@ class BaseScript(ArgumentParserScript, BaseScriptProtocol):
             units=units,
             **kwargs,
         )
-        logger.info("Scheduling a job:\n%s", schedule_kwargs)
+        logger.debug("Scheduling a job:\n%s", schedule_kwargs)
         try:
             job = project.jobs.run(**schedule_kwargs)
         except DuplicateJobError as e:
@@ -362,7 +370,9 @@ class BaseScript(ArgumentParserScript, BaseScriptProtocol):
                 break
             kwargs["start"] += count
 
-    def get_jobs_with_tags(self, spider, tags, project_id=None, **kwargs) -> Generator[Job, None, None]:
+    def get_jobs_with_tags(
+        self, spider: str, tags: List[str], project_id: Optional[int] = None, **kwargs
+    ) -> Generator[Job, None, None]:
         """
         Get jobs with target tags
         """
@@ -418,16 +428,25 @@ class BaseLoopScriptProtocol(BaseScriptProtocol, Protocol):
         """
         ...
 
+    @abc.abstractmethod
+    def _run_loops(self) -> Generator[bool, None, None]:
+        ...
+
+    def base_loop_tasks(self):
+        ...
+
 
 class BaseLoopScript(BaseScript, BaseLoopScriptProtocol):
 
     # If 0, don't loop. If positive number, repeat loop every given number of seconds
     # --loop-mode command line option overrides it
     loop_mode = 0
+    max_running_time = 0
 
     def __init__(self):
         self.workflow_loop_enabled = False
         super().__init__()
+        self.__start_time = time.time()
 
     def add_argparser_options(self):
         super().add_argparser_options()
@@ -439,30 +458,43 @@ class BaseLoopScript(BaseScript, BaseLoopScriptProtocol):
             metavar="SECONDS",
             default=self.loop_mode,
         )
+        self.argparser.add_argument(
+            "--max-running-time",
+            type=int,
+            default=self.max_running_time,
+            metavar="SECONDS",
+            help="Set max time after which the script will auto shutdown.",
+        )
 
     def on_start(self):
         pass
 
     def run(self):
         self._on_start()
-        for _ in self._run_loops():
-            pass
+        for loop_result in self._run_loops():
+            if loop_result and self.args.loop_mode:
+                time.sleep(self.args.loop_mode)
+            else:
+                self._close()
+                logger.info("No more tasks")
+                break
 
-    def _base_loop_tasks(self):
+    def base_loop_tasks(self):
         pass
 
+    def __base_loop_tasks(self):
+        self.base_loop_tasks()
+        if self.args.max_running_time and time.time() - self.__start_time > self.args.max_running_time:
+            logger.info("Time limit reached. Closing.")
+            self.workflow_loop_enabled = False
+
     def _run_loops(self) -> Generator[bool, None, None]:
-        while self.workflow_loop_enabled:
-            self._base_loop_tasks()
+        while True:
+            self.__base_loop_tasks()
+            if not self.workflow_loop_enabled:
+                break
             try:
-                loop_result = self.workflow_loop()
-                yield loop_result
-                if loop_result and self.args.loop_mode:
-                    time.sleep(self.args.loop_mode)
-                else:
-                    self._close()
-                    logger.info("No more tasks")
-                    break
+                yield self.workflow_loop()
             except KeyboardInterrupt:
                 logger.info("Bye")
                 break
@@ -476,3 +508,54 @@ class BaseLoopScript(BaseScript, BaseLoopScriptProtocol):
 
     def _close(self):
         self.on_close()
+
+
+class BaseLoopScriptAsyncMixin(BaseLoopScriptProtocol):
+    @dash_retry_decorator
+    async def _async_schedule_job(
+        self, spider: str, tags=None, units=None, project_id=None, **kwargs
+    ) -> Optional[JobKey]:
+        project = self.get_project(project_id)
+        schedule_kwargs = dict(
+            spider=spider,
+            add_tag=self._make_children_tags(tags),
+            debugunits=units,
+            **kwargs,
+        )
+        logger.debug("Scheduling a job with jobargs:\n%s", schedule_kwargs)
+        try:
+            loop = asyncio.get_event_loop()
+            future = loop.run_in_executor(None, partial(project.jobs.run, **schedule_kwargs))
+            job = await future
+        except DuplicateJobError as e:
+            logger.error(str(e))
+        except Exception:
+            raise
+        else:
+            logger.info(f"Scheduled job {job.key}.")
+            return job.key
+        return None
+
+    async def async_schedule_spider(
+        self,
+        spider: str,
+        tags: Optional[List[str]] = None,
+        units: Optional[int] = None,
+        project_id: Optional[int] = None,
+        **kwargs,
+    ) -> Optional[JobKey]:
+        return await self._async_schedule_job(spider=spider, tags=tags, units=units, project_id=project_id, **kwargs)
+
+    async def _async_run_loops(self) -> AsyncGenerator[bool, None]:
+        for result in self._run_loops():
+            yield await cast(Awaitable[bool], result)
+
+    async def run(self):
+        self._on_start()
+        async for loop_result in self._async_run_loops():
+            if loop_result and self.args.loop_mode:
+                await asyncio.sleep(self.args.loop_mode)
+            else:
+                self._close()
+                logger.info("No more tasks")
+                break
