@@ -3,18 +3,17 @@ import json
 import os
 import hashlib
 from typing import Optional
-from traceback import format_tb
 
-from retrying import retry
+from tenacity import retry, retry_if_exception_type, before_sleep_log, stop_after_attempt, wait_fixed
 from scrapinghub.client.exceptions import ServerError
-from requests.exceptions import ReadTimeout, ConnectionError
+from requests.exceptions import ReadTimeout, ConnectionError, HTTPError
 
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.INFO)
 
 
-def hashstr(text):
+def hashstr(text: str) -> str:
     u = hashlib.sha1()
     u.update(text.encode("utf8"))
     return u.hexdigest()
@@ -35,8 +34,8 @@ def resolve_project_id(project_id=None) -> Optional[int]:
         return int(project_id)
 
     # read from environment
-    if os.environ.get("PROJECT_ID"):
-        return int(os.environ.get("PROJECT_ID"))
+    if os.environ.get("PROJECT_ID") is not None:
+        return int(os.environ["PROJECT_ID"])
 
     # for ScrapyCloud jobs:
     if os.environ.get("SHUB_JOBKEY"):
@@ -63,17 +62,12 @@ MINS_IN_A_DAY = 24 * 60
 ONE_MIN_IN_S = 60
 
 
-def just_log_exception(exception):
-    logger.error("".join(format_tb(exception.__traceback__)[1:]))
-    logger.error(repr(exception))
-    if isinstance(exception, (ServerError, ReadTimeout, ConnectionError)):
-        logger.info("Waiting %d seconds", ONE_MIN_IN_S)
-        return True
-    return False
-
-
 dash_retry_decorator = retry(
-    retry_on_exception=just_log_exception, wait_fixed=ONE_MIN_IN_S * 1000, stop_max_attempt_number=MINS_IN_A_DAY
+    retry=retry_if_exception_type((ServerError, ReadTimeout, ConnectionError, HTTPError)),
+    before_sleep=before_sleep_log(logger, logging.ERROR),
+    reraise=True,
+    stop=stop_after_attempt(MINS_IN_A_DAY),
+    wait=wait_fixed(ONE_MIN_IN_S),
 )
 
 
